@@ -1,7 +1,7 @@
 context("fmode")
 
 # rm(list = ls())
-
+set.seed(101)
 x <- round(abs(10*rnorm(100)))
 w <- as.integer(round(abs(10*rnorm(100)))) # round(abs(rnorm(100)), 1) -> Numeric precision issues in R
 xNA <- x
@@ -12,7 +12,7 @@ f <- as.factor(sample.int(10, 100, TRUE))
 data <- wlddev[wlddev$iso3c %in% c("BLZ","IND","USA","SRB","GRL"), ]
 l <- nrow(data)
 g <- GRP(droplevels(data$iso3c))
-gf <- as.factor_GRP(g)
+gf <- as_factor_GRP(g)
 dataNA <- na_insert(data)
 m <- as.matrix(num_vars(data)) # without num_vars also works for ties = "first"
 mNA <- as.matrix(num_vars(dataNA))
@@ -40,7 +40,7 @@ minwa <- function(x) {
       if(!length(xna)) return(`attributes<-`(NA_real_, attributes(x)))
     }
   }
-  `attributes<-`(`storage.mode<-`(min(xna), storage.mode(x)), attributes(x))
+  `attributes<-`(`storage.mode<-`(base::min(xna), storage.mode(x)), attributes(x))
 }
 maxwa <- function(x) {
   xna <- unattrib(x)
@@ -48,7 +48,7 @@ maxwa <- function(x) {
     xna <- na_rm(xna)
     if(!length(xna)) return(`attributes<-`(NA_real_, attributes(x)))
   }
-  `attributes<-`(`storage.mode<-`(max(xna), storage.mode(x)), attributes(x))
+  `attributes<-`(`storage.mode<-`(base::max(xna), storage.mode(x)), attributes(x))
 }
 
 if(identical(Sys.getenv("NCRAN"), "TRUE")) {
@@ -100,9 +100,9 @@ wMode <- function(x, w, na.rm = FALSE, ties = "first") {
   g <- GRP.default(x, call = FALSE)
   switch(ties,
          first = {
-           g <- as.factor_GRP(g)
+           g <- as_factor_GRP(g)
            o <- radixorder(unlist(split.default(seq_along(w), g), use.names = FALSE))
-           sw <- unlist(lapply(split.default(w, g), cumsum), use.names = FALSE)[o]
+           sw <- unlist(lapply(split.default(w, g), base::cumsum), use.names = FALSE)[o]
            fsubset.default(x, which.max(sw))
          },
          min = minwa(fsubset.default(g[["groups"]][[1L]], whichmax(fsum.default(w, g, use.g.names = FALSE)))),
@@ -118,7 +118,7 @@ wBY <- function(x, f, FUN, w, ...) {
   if(is.atomic(x)) return(dapply(x, function(xi) mapply(FUN, split(xi, f), wspl, ...)))
   dapply(dapply(x, function(xi) {
     r <- Map(FUN, split(xi, f), wspl, ...)
-    if(is.Date(xi)) do.call(c, r) else unlist(r)
+    if(is_date(xi)) do.call(c, r) else unlist(r)
     }), `names<-`, NULL)
 }
 
